% This is a sample driver that computes the forward problem for EIT

% make sure that mesh generation code that we use is in Matlab's search path
addpath distmesh

% Generate the grid 
grid = gengrid(32,0.1);
p=grid.p; np=size(p,1);
t=grid.t; nt=size(t,1);
be=grid.be; nbe=size(be,1);
% Note: the grid has been already generated (grid32.mat) and can be accessed by
% replacing the grid = gengrid(...); line above by 
% load grid32

% ** True conductivity (evaluated at the triangle centers)
c=triangle_centers(grid);
truecond = phantom1(c(:,1),c(:,2));
% plot true conductivity
figure(1);
trisurf(t,p(:,1),p(:,2),zeros(size(p,1),1),truecond); 
view(2); axis equal; colorbar
title('true conductivity'); xlabel('x'); ylabel('y');

% assemble system 
A = stiffmat(truecond,grid);
% change last equation to impose grounding condition: that the integral over
% the boundary of the potential should be 0
A(end,:) = 0; A(end,1:nbe) = boundary_int(grid);

% assemble right hand side (the current patterns are predefined)
I = currents(grid);
I(end,:) = 0; % to go with last equation above

% solve the system
u=A\I;

% animate the potentials, if you look very hard it is possible to make up the
% conductivity from the gradient of these potentials. However, you have only
% access to boundary data
for j=1:nbe,
 trisurf(t,p(:,1),p(:,2),u(:,j)); 
 axis([-1 1 -1 1 -0.05 0.25]);
 pause(0.1);
end;

% try out  the following identy with gradients. This is the identity that 
% we use to get the adjoint state method working
A = stiffmat(truecond,grid);
[Gx,Gy]=gradmat(grid);
S = spdiags(truecond.*triangle_area(grid),0,nt,nt);
Ap = Gx'*S*Gx + Gy'*S*Gy;
fprintf('should be small=%d\n',norm(A-Ap,'fro'));
