% function [f,g,Hinfo] = fobj(sigma,pars)
%
% objective function
%
% inputs
%   sigma 	conductivity (values at the center of triangles)
%   pars	contains several precomputed quantities for the grid
%
% outputs
%   f 		value of objective function at sigma
%   g		gradient of objective function (not implemented)
%   Hinfo       data structure with information to compute Hesssian

% Fernando Guevara Vasquez 2008
function [f,g,Hinfo] = fobj(sigma,pars)
grid=pars.grid;
nt = size(grid.t,1);
nb = size(grid.be,1);

% assemble system 
A = stiffmat(sigma,grid);

% change last equation to impose grounding condition: that the integral over
% the boundary of the potential should be 0
A(end,:) = 0; A(end,1:nb) = pars.bint; 

% assemble right hand side (the current patterns are predefined)
u = A\pars.I;

% compute residual
r = pars.C*u - pars.V;

% this is the objective function
f= pars.alpha*sigma'*(triangle_area(grid).*sigma); % regularization term
for i = 1:nb,
 f = f + 0.5*r(:,i)'*pars.Qbdry*r(:,i);
end;

if (nargout>1)
 % compute gradient using adjoint state method + gradient of regularization term
 %       g =    ........
end;

if (nargout>2)
 % compute Hinfo
 % Hinfo.A = ...
end;
