% function H = fobj_GNHessian(Hinfo,pars)
%
% Computes the Gauss-Newton approximation to the Hessian 
%
% This can only be done for relatively small problems, but it is helpful for
% testing your own GN Hessian times vector routine.
%
% Inputs
%   Hinfo  data structure containing information about how to compute the
%          Hessian. In this case it contains the forward problem solution and
%          the stiffness matrix.
%
%   pars same data structure that is given to fobj you may have to adjust the
%        code to your own data structure.
%
% Outputs
%   H    Gauss-Newton approximation to the Hessian of fobj

% Fernando Guevara Vasquez 2008
function H = fobj_GNHessian(Hinfo,pars)
% first compute the Jacobian of the mapping conductivity -> measurements
% column by column
p = pars.grid.p; t = pars.grid.t;
np = size(p,1); nt = size(t,1);
nb = size(pars.grid.be,1);
J = zeros(nb^2,nt);
for i=1:nt,
 dsig = sparse(nt,1); dsig(i)=1;
 Adsig = stiffmat(dsig,pars.grid);
 du = - Hinfo.A\(Adsig*Hinfo.u);
 % restrict to the boundary
 du=pars.C*du;
 % collect all measurements in one single vector of size (# boundary nodes)^2
 J(:,i) = du(:);

 % this is a lengthy calculation, display a progress bar
 if ( mod(i,10)==0) fprintf('.'); end;
end;
fprintf('\n');

% Form the Gauss-Newton approximation to the Hessian
% note: doing J'*J does not take into account the right inner prod
%       at the boundary.
H = J'*kron(eye(nb),pars.Qbdry)*J; 


% Hessian of regularization terms etc...
% H  = H + ...
