% function [Gx,Gy] = gradmat(grid)
%
%    Computes matrices Gx,Gy for a given grid s.t. 
%
%    if u is a function defined on the nodes of the triangulation, then:
%
%       Gx*u is the vector containing the partial wrt x derivative of u
%            (since we use P1 elements this will be constant on each triangle
%
%       Gy*u is the vector containing the partial wrt y derivative of u
%
%    inputs
%     grid structure containing grid info. See gengrid.m
%
%    outputs
%     Gx   discretization of \partial/\partial x on grid with P1 elements
%     Gy   discretization of \partial/\partial y on grid with P1 elements
%
%    For details on the computation of the gradients see:
%    J. Alberty, C. Carstensen and S. A. Funken  02-11-99
%    "Remarks around 50 lines of Matlab: Short finite element 
%    implementation"

% Fernando Guevara Vasquez
function [Gx,Gy] = gradmat(grid)
t = grid.t; p = grid.p;
% Assembly of the gradients
Gx = sparse(size(t,1),size(p,1));
Gy = sparse(size(t,1),size(p,1));
for j = 1:size(t,1),
 G = grada3(p(t(j,:),:));
 Gx(j,t(j,:)) = Gx(j,t(j,:)) + G(:,1)';
 Gy(j,t(j,:)) = Gy(j,t(j,:)) + G(:,2)';
end;

function [G]= grada3(vertices)
G = [ones(1,3);vertices'] \ [0,0;1,0;0,1];
