% function M = massmat(grid)
%
%  Mass matrix
%
% inputs
%   grid structure containing grid info. See gengrid.m.
%
% outputs
%   M   nnodes x nnodes matrix with entries being the integrals
%       int_{domain} phi_i(x,y) phi_j(x,y) dx dy
%
%    Adapted from:
%    J. Alberty, C. Carstensen and S. A. Funken  02-11-99
%    "Remarks around 50 lines of Matlab: Short finite element 
%    implementation"

function M = massmat(grid)
p = grid.p; t = grid.t;
% Mass matrix 
M = sparse(size(t,1),size(t,1));
for j = 1:size(t,1)
  M(t(j,:),t(j,:)) = M(t(j,:),t(j,:)) ...
      + det([1,1,1;p(t(j,:),:)'])*[2,1,1;1,2,1;1,1,2]/24;
end
