% function res = phantom1(x,y,pars)
% Phantom chest example similar to,
%   Knudsen, Mueller, Siltanen
%   Numerical solution method for the dbar-equation in the plane
%   Journal of Computational Physics 198 (2004), 500-517.
%
% conductivities scaled to 1 with the background are:
%   - background: 1
%   - lungs:      1/3
%   - heart:      2
%
% the domain of definition is the unit circle.
%
% inputs
%   x,y   vectors of same length where to evaluate the conductivity
%   pars   ignored
% outputs
%   res   conductivity values at the given points
  
% Fernando Guevara Vasquez 2008
function res = phantom1(x,y,pars)
theta1= pi/6; x1= 0.5; y1= 1/3; a1=0.3;  b1=0.55;
theta2=-pi/6; x2=-0.5; y2= 1/3; a2=0.3;  b2=0.55;
theta3=    0; x3=   0; y3=-1/2; a3=1/4;  b3=1/4;

res = 1 ...
    + (1/3-1)*isinellipse(x,y,theta1,a1,b1,x1,y1) ...
    + (1/3-1)*isinellipse(x,y,theta2,a2,b2,x2,y2) ...
    + (2  -1)*isinellipse(x,y,theta3,a3,b3,x3,y3);

function res=isinellipse(x,y,theta,a,b,x0,y0)
D = diag([1/a^2,1/b^2]); Q = [cos(theta), sin(theta)
                      -sin(theta), cos(theta) ];
A = Q'*D*Q;
res = zeros(prod(size(x)),1);
for i=1:prod(size(x)),
 v = [x(i)-x0;y(i)-y0];
 res(i)=v'*A*v<1;
end;
res=reshape(res,size(x));
