% tests tvfun.m by generating a random triangular mesh of n points
n=50; beta=1;
p = randn(n,2); t = delaunay(p(:,1),p(:,2));
grid.p = p; grid.t = t;

% compute discrete gradient matrices for grid
[Dx,Dy]=gradmat(grid);

% weights (areas of each triangle, needed for the integration to make sense)
opts.wts=triangle_area(grid); 
% derivative operator in x and in y
opts.Dx=Dx; opts.Dy=Dy; 
% psi is an approximation to 2*abs(t) = 2*sqrt(t^2), and derivatives
opts.psi=  inline(sprintf('2*sqrt(t+%d^2)',beta));
opts.dpsi= inline(sprintf('1./sqrt(t+%d^2)',beta));
opts.d2psi=inline(sprintf('(-1/2)./((x+%d^2).^(3/2))',beta));

sig0=randn(n,1);
dsig=randn(n,1);

[J0,DJ0,Hinfo]=tvfun(sig0,opts);
fprintf('---------------------------- first order test:\n')
for scale=10.^(0:-1:-10),
 [J1]=tvfun(sig0+scale*dsig,opts);
 fprintf('scale=%4.1e, ( first order residual ) / scale^2 = %d\n',scale,...
          abs(J1-J0-scale*DJ0'*dsig)/scale^2);
end;

fprintf('---------------------------- second order test:\n')
for scale=10.^(0:-1:-10),
 [J1]=tvfun(sig0+scale*dsig,opts);
 Hv  =tvfun_hessmult(Hinfo,dsig,opts);
 fprintf('scale=%4.1e, ( second order residual ) / scale^3 = %d\n',scale,...
          abs(J1-J0-scale*DJ0'*dsig - scale^2*dsig'*Hv/2)/scale^3);
end;

