% function a = triangle_area(grid)
%
% gives the area of each triangle in grid
%
% 
% inputs
%   grid  structure containing grid info. See gengrid.m
%
% outputs
%   a     vector of length number of triangles containing the area of each triangle
%         in the discretization
%
% Example use
%
% Let f be a vector with the values of some function at the triangle centers
% then a'*f approximates int_{domain} f(x) dx
function a = triangle_area(grid)
t=grid.t; nt = size(t,1);
p=grid.p;
a = zeros(nt,1);
for j=1:nt,
 a(j) = det([1 1 1; p(t(j,:),:)'])/2;
end;
