% function [J,DJ,D2J] = tvfun(f,opts)
%   .wts     Hilbert space weights (area of each triangle, mass matrix)
%   .Dx      derivative in x operator (a sparse matrix)
%   .Dy      derivative in y operator (a sparse matrix)
%   .psi     psi(x)   approximates 2*sqrt(x)
%   .dpsi    dpsi(x)  is psi'(x)
%   .d2psi   d2psi(x) is psi''(x)
% See Vogel, Computational methods for inverse problems, Chap. 8

% Fernando Guevara Vasquez 2008
function [J,DJ,Hinfo] = tvfun(f,opts)
Dxf=opts.Dx*f; Dyf=opts.Dy*f;
norm2Df=Dxf.^2 + Dyf.^2;
% computes TV functional
J= (1/2)*sum(opts.wts.*feval(opts.psi,norm2Df));
if (nargout>1)
 % computes gradient of TV functional
 dpsif=feval(opts.dpsi,norm2Df);
 DJ= opts.Dx'*(opts.wts.*dpsif.*Dxf) + opts.Dy'*(opts.wts.*dpsif.*Dyf);
 if (nargout>2)
  % gives some Hessian info to be used in tvfun_hessmult.m
  Hinfo.f    =f;
  Hinfo.dpsif=dpsif;
  Hinfo.Dxf  =Dxf;
  Hinfo.Dyf  =Dyf;
 end;
end;
