% function H = tvfun_hess(Hinfo,opts)
%  computes  H = D^2J(f) where J is TV functional
%
% See Vogel, Computational methods for inverse problems, Chap. 8
% and tvfun.m

% Fernando Guevara Vasquez 2008

function [H] = tvfun_hess(Hinfo,opts)
dpsif=Hinfo.dpsif; Dxf=Hinfo.Dxf; Dyf=Hinfo.Dyf; f=Hinfo.f;

% L(f) v part
N=length(opts.wts);
D1=spdiags(opts.wts.*dpsif,0,N,N);
H = opts.Dx'*D1*opts.Dx + opts.Dy'*D1*opts.Dy;


% ( L'(f) f ) v part
quasinewton=0;
if (isfield(opts,'QN')) quasinewton=opts.QN; end;
if (~quasinewton)
d2psif=2*feval(opts.d2psi,Dxf.^2 + Dyf.^2);
diagxx=spdiags(opts.wts.* Dxf.^2  .* d2psif,0,N,N);
diagxy=spdiags(opts.wts.* Dxf.*Dyf.* d2psif,0,N,N);
diagyy=spdiags(opts.wts.* Dyf.^2  .* d2psif,0,N,N);
H = H +   opts.Dx' * diagxx * opts.Dx  ...  % x,x
      +   opts.Dx' * diagxy * opts.Dy  ...  % x,y
      +   opts.Dy' * diagxy * opts.Dx  ...  % y,x
      +   opts.Dy' * diagyy * opts.Dy ;     % y,y
end;
