% function y = tvfun_hessmult(Hinfo,v,opts)
%  computes  y = D^2J(f) v where J is TV functional
% 
% See Vogel, Computational methods for inverse problems, Chap. 8
% and tvfun.m

% Fernando Guevara Vasquez 2008

function [y] = tvfun_hessmult(Hinfo,v,opts)
dpsif=Hinfo.dpsif; Dxf=Hinfo.Dxf; Dyf=Hinfo.Dyf; f=Hinfo.f;
Dxv = opts.Dx*v; Dyv = opts.Dy*v;

% L(f) v part
N=length(opts.wts);
D1=spdiags(opts.wts.*dpsif,0,N,N);
y = opts.Dx'*(D1*Dxv) + opts.Dy'*(D1*Dyv);


% ( L'(f) f ) v part
quasinewton=0;
if (isfield(opts,'QN')) quasinewton=opts.QN; end;
if (~quasinewton)
d2psif=feval(opts.d2psi,Dxf.^2 + Dyf.^2);
diagxx=spdiags(opts.wts.* Dxf.^2  .* d2psif,0,N,N);
diagxy=spdiags(opts.wts.* Dxf.*Dyf.* d2psif,0,N,N);
diagyy=spdiags(opts.wts.* Dyf.^2  .* d2psif,0,N,N);
y = y +   2*opts.Dx' *( diagxx * Dxv ) ...  % x,x
      +   2*opts.Dx' *( diagxy * Dyv ) ...  % x,y
      +   2*opts.Dy' *( diagxy * Dxv ) ...  % y,x
      +   2*opts.Dy' *( diagyy * Dyv );     % y,y
end;
