% function opts=tvfuninit(grid,beta)
%
%  Initializes data structure to use in (smoothed) TV functional
%  see tvfun.m
%
%  inputs
%    grid   grid data structure
%    beta   parameter controlling smoothness of the TV functional.
%           the higher this is the smoother the tip of the the absolute
%           value approximation we use is.
%  outputs
%    opts   data structure needed by tvfun.m, contains information about
%           how to compute the gradient etc...

% Fernando Guevara Vasquez 2008
function opts=tvfuninit(grid,beta)
% Do a Delaunay triangulation of the triangle centers,
% this gives us a "dual grid" dgrid on which to compute "gradients" of the conductivity
c = triangle_centers(grid);
t = delaunay(c(:,1),c(:,2));
dgrid.p=c; dgrid.t=t;
[Gx,Gy] = gradmat(dgrid);


% weights (areas of each triangle, needed for the integration to make sense)
opts.wts=triangle_area(dgrid);
% derivative operator in x and in y
opts.Dx=Gx; opts.Dy=Gy;
% psi is an approximation to 2*abs(t) = 2*sqrt(t^2), and derivatives
psi=  @(t,beta) 2*sqrt(t+beta^2);
dpsi= @(t,beta) 1./sqrt(t+beta^2);
d2psi=@(t,beta) (-1/2)./((t+beta^2).^(3/2));

opts.psi=  @(t) psi(t,beta);
opts.dpsi= @(t) dpsi(t,beta);
opts.d2psi=@(t) d2psi(t,beta);
